--[[
    Stable by LordFarlander
    Version 2.0
    $Revision: 76975 $

    Based on Kennel.
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = Rock:GetLibrary( "LibRockLocale-1.0" ):GetTranslationNamespace( "Stable" );

Stable = Rock:NewAddon( "Stable", "LibRockConsole-1.0", "LibRockEvent-1.0", "LibRockDB-1.0", "LibRockConfig-1.0" );
Stable.version = tonumber( string.match( "$Rev: 76975 $", "(%d+)" ) );
Stable:SetDatabase( "StableDB", "StableDBPC" );
Stable:SetDatabaseDefaults( "char", {
        DoSwap = {
            ["MiniPets"] = true, ["MiniPets.Normal"] = true, ["MiniPets.Reagented"] = true, ["MiniPets.Equipment"] = true,
            ["Mounts"] = true,
            ["Mounts.Ground"] = true, ["Mounts.Ground.Slow"] = true, ["Mounts.Ground.Slow"] = true, ["Mounts.Ground.Fast"] = true, ["Mounts.Ground.VeryFast"] = true,
            ["Mounts.Flying"] = true, ["Mounts.Flying.Fast"] = true, ["Mounts.Flying.VeryFast"] = true, ["Mounts.Flying.ExtremelyFast"] = true, },
        } );

local PetAndMountDatabase = Rock:GetLibrary( "PetAndMountDatabase-1.0", false, true );
if( not PetAndMountDatabase ) then
    error( "Stable requires PetAndMountDatabase-1.0" );
    return;
end--if

Stable.bankopen = false;
Stable.swapped = false;
Stable.sets = {};

function Stable:OnEnable()
    local Sets = { PetAndMountDatabase:GetSubsets( "MiniPets" ) };
    local MountSubsets = PetAndMountDatabase:GetSubsets( "Mounts" );
    local AllSets = {};
    local options = { type = "group", args = {}, name = L["Stable"], desc = L["Swap a random minipet and/or mount every time you visit the bank"], };

    self:AddEventListener( "BANKFRAME_OPENED" );
    self:AddEventListener( "BANKFRAME_CLOSED",
        function()
            if( ( not Stable.bankopen ) and ( not Stable.swapped ) ) then
                Stable:BANKFRAME_OPENED();
            end--if
            if( Stable.bankopen and Stable.swapped and FuBar_CorkFu ) then
                FuBar_CorkFu:GetModule( "Minipet" ):ActivatePet();
            end--if
            Stable.bankopen = false;
        end );

    for _, set in pairs( MountSubsets ) do
        table.insert( Sets, PetAndMountDatabase:GetSubsets( set ) );
    end--for
    -- We want to make sure not to swap Quests and Children and remove the base set
    for _, set in pairs( Sets ) do
        self.CheckSets( set, AllSets );
    end--for
    for _, set in pairs( AllSets ) do
        local setName = set;
        local setParts = { strsplit( ".", set ) };
        local commandString = L["swap"];
        local shortString = "";
        local longString = "";
        local stableSets = self.sets;
        local commandSet = "";
        local setPartsCount = #setParts;

        if( setPartsCount > 1 ) then
            setName = setParts[1];
        end--if
        if( stableSets[setName] == nil ) then
            stableSets[setName] = {};
        end--if
        stableSets[setName][set] = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 } };
        if( setPartsCount >= 3 ) then
            stableSets[setName][set].subSet = table.concat( setParts, ".", 1, setPartsCount - 1 );
        end--if
        -- Build commands
        for _, setSubname in pairs( setParts ) do
            local capName = setSubname;
            local lowerName = setName:lower();

            if( L:HasTranslation( setSubname .. "_Cap" ) ) then
                capName = L[setSubname .. "_Cap"];
            end--if
            if( L:HasTranslation( setSubname .. "_NoCap" ) ) then
                lowerName = L[setSubname .. "_NoCap"];
            end--if
            commandString = commandString .. capName;
            if( shortString:len() > 0 ) then
                shortString = " " .. shortString;
            end--if
            shortString = capName .. shortString;
            if( longString:len() > 0 ) then
                longString = " " .. longString;
            end--if
            longString = lowerName .. longString;
            if( commandSet:len() > 0 ) then
                commandSet = commandSet .. ".";
            end--if
            commandSet = commandSet .. setSubname;
            if( options.args[commandString] == nil ) then
                options.args[commandString] = {
                    type = "boolean",
                    name = L["Swap %s"]:format( shortString ),
                    desc = L["Toggles the swapping of %s."]:format( longString ),
                    get = loadstring( ("return Stable.db.char.DoSwap[%q];"):format( commandSet ) ),
                    set = loadstring( ("Stable.db.char.DoSwap[%q] = not Stable.db.char.DoSwap[%q];"):format( commandSet, commandSet ) ),
                    };
            end--if
        end--for
    end--for
    self:SetConfigSlashCommand( "/stable", "/stbl" );
    self:SetConfigTable( options );

    --Upgrade check
    --Convert old save values
    if( self.db.char.DoSwap["GroundMounts"] ~= nil ) then
        self.db.char.DoSwap["Mounts.Ground"] = self.db.char.DoSwap["GroundMounts"];
        self.db.char.DoSwap["GroundMounts"] = nil;
    end--if
    if( self.db.char.DoSwap["FlyingMounts"] ~= nil ) then
        self.db.char.DoSwap["Mounts.Flying"] = self.db.char.DoSwap["FlyingMounts"];
        self.db.char.DoSwap["FlyingMounts"] = nil;
    end--if
    if( self.db.char.DoSwap["MiniPets.Holiday"] ~= nil ) then
        self.db.char.DoSwap["MiniPets.Reagented"] = self.db.char.DoSwap["MiniPets.Holiday"];
        self.db.char.DoSwap["MiniPets.Holiday"] = nil;
    end--if
    if( self.db.char.LastVersion ~= self.version ) then
        self.db.char.LastVersion = self.version;
    end--if
end--Stable:OnEnable()

function Stable.CheckSets( sets, dst )
    for _, name in pairs( sets ) do
        if( ( not name:find( ".Quest" ) ) and ( not name:find( ".Children" ) ) ) then
            local start, ends = name:find( "PetAndMountDatabase." );
            
            if( ends ) then
                name = name:sub( ends + 1 );
            end--if
            table.insert( dst, name );
        end--if
    end--for
end--Stable.CheckSets( sets, dst )

function Stable:BANKFRAME_OPENED()
    local doSwap = self.db.char.DoSwap;
    local stableSets = self.sets;

    self.bankopen = true;
    self.swapped = false;

    for _, set in pairs( stableSets ) do
       for _, subset in pairs( set ) do
           subset.player = { bag = {}, slot = {}, num = 0 };
           subset.bank = { bag = {}, slot = {}, num = 0 };
       end--for
    end--for
    for bag = BANK_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
        for slot = 1, GetContainerNumSlots( bag ) do
            local itemLink = GetContainerItemLink( bag, slot );
            local bagSet = ( bag <= NUM_BAG_SLOTS and bag >= 0 ) and "player" or "bank";

            if( itemLink ) then
                for setName, set in pairs( stableSets ) do
                    if( doSwap[setName] ) then
                        for subsetName, subset in pairs( set ) do
                            local doSwapSet = doSwap[subsetName];

                            if( subset.subSet ) then
                                doSwapSet = doSwapSet and doSwap[subset.subSet];
                            end--if                        
--                            if( itemLink and doSwap[subset.set] and ( ( ( swappingMiniPets and bit.band( GetItemFamily( itemLink ), 4096 ) ~= 0 ) ) or PetAndMountDatabase:ItemInSet( itemLink, subset.set ) ) ) then
                            if( doSwapSet and PetAndMountDatabase:ItemInSet( itemLink, subsetName ) ) then
                                local petSet = subset[bagSet];
                                local n = petSet.num + 1;
        
                                petSet.num = n;
                                petSet.bag[n] = bag;
                                petSet.slot[n] = slot;
                            end--if
                        end--for
                    end--if
                end--for
            end--if
        end--for
    end--for
    for setName, set in pairs( stableSets ) do
        if( doSwap[setName] ) then
            for subsetName, subset in pairs( set ) do
                local doSwapSet = doSwap[subsetName];

                if( set.subSet ) then
                    doSwapSet = doSwapSet and doSwap[set.subSet];
                end--if                
                if( doSwapSet ) then
                    self.swapped = self:DoSwaps( subset ) or self.swapped;
                end--if
            end--for
        end--if
    end--for
end--Stable:BANKFRAME_OPENED()

------------------------------
--     Helper Functions     --
------------------------------
function Stable:DoSwaps( set )
    local p, b = set.player, set.bank;

    if( ( p.num == 0 ) or ( b.num == 0 ) ) then
        return false;
    end--if
    for i = 1, p.num do
        local r = math.random( b.num + 1 - i );

        PickupContainerItem( p.bag[i], p.slot[i] );
        PickupContainerItem( b.bag[r], b.slot[r] );
        table.remove( b.bag, r );
        table.remove( b.slot, r );
    end--for
    return ( p.num > 0 ) and ( b.num > 0 );
end--Stable:DoSwaps( set )
